try:
    # Python3.8+ standard library
    from importlib.metadata import (  # pyright: ignore[reportMissingImports]
        PackageNotFoundError,
        version,
    )
except ImportError:
    # Python3.7 module backport
    from importlib_metadata import (
        PackageNotFoundError,
        version,  # pyright: ignore[reportMissingImports]
    )

__version__: str
try:
    __version__ = version(__package__ or __name__)
except PackageNotFoundError:
    # Could fail to find package when developing locally.
    # Try reading from pyproject.toml as a backup.

    from pathlib import Path

    try:
        # Python >= 3.11 standard library
        from tomllib import load as load_toml  # pyright: ignore[reportMissingImports]
    except (ImportError, ModuleNotFoundError):
        # Python <= 3.10 predecessor library
        from tomli import load as load_toml  # pyright: ignore[reportMissingImports]

    try:
        with open(Path(__file__).resolve().parents[1] / "pyproject.toml", "rb") as f:
            pyproject = load_toml(f)
            __version__ = pyproject["tool"]["poetry"]["version"]
    except OSError:
        # File IO error
        __version__ = "local"



import os
import sys
import logging
import ruamel.yaml

from pydantic import BaseModel, ValidationError

from mxcubeweb.core.models.configmodels import (
    UIPropertiesListModel,
    AppConfigModel,
    FlaskConfigModel,
    MXCUBEAppConfigModel,
)


class ConfigLoader:
    @staticmethod
    def load(path: str, schema: BaseModel):
        with open(os.path.join(path)) as f:
            config = ruamel.yaml.load(f.read(), ruamel.yaml.RoundTripLoader)
            try:
                model = schema.parse_obj(config)
            except ValidationError:
                logging.getLogger("HWR").error(f"Validation error in {path}:")
                logging.getLogger("HWR").exception("")
                sys.exit(-1)

        return model


class Config:
    CONFIG_ROOT_PATH: str = ""

    flask: FlaskConfigModel
    app: MXCUBEAppConfigModel

    def __init__(self, fpath):
        Config.CONFIG_ROOT_PATH = fpath
        app_config = self.load_config("server", AppConfigModel)
        uiprop = self.load_config("ui", UIPropertiesListModel)

        self.flask = app_config.server
        self.app = app_config.mxcube
        self.app.ui_properties = uiprop

    def load_config(self, component_name, schema):
        fpath = os.path.join(Config.CONFIG_ROOT_PATH, f"{component_name}.yaml")
        config = ConfigLoader().load(path=fpath, schema=schema)

        return config



# -*- coding: utf-8 -*-
import io

from flask import Blueprint, Response, jsonify, request, send_file


def init_route(app, server, url_prefix):
    bp = Blueprint("workflow", __name__, url_prefix=url_prefix)

    @bp.route("/", methods=["GET"])
    @server.restrict
    def workflow():
        return jsonify(app.workflow.get_available_workflows())

    @bp.route("/", methods=["POST"])
    @server.restrict
    def submit_parameters():
        data = request.get_json()
        app.workflow.submit_parameters(data)
        return Response(status=200)

    @bp.route("/gphl", methods=["POST"])
    @server.restrict
    def submit_gphl_parameters():
        data = request.get_json()
        app.workflow.update_gphl_parameters(data)
        return Response(status=200)

    @bp.route("/mesh_result/<gid>/<t>", methods=["GET"])
    # @server.restrict
    def get_grid_data(gid, t, rand):
        res = send_file(
            io.BytesIO(app.workflow.get_mesh_result(gid, t)),
            mimetype="image/png",
        )

        return res

    # This route is only for testing
    @bp.route("/dialog/<wf>", methods=["GET"])
    @server.restrict
    def workflow_dialog(wf):
        dialog = app.workflow.test_workflow_dialog(wf)
        server.emit("workflowParametersDialog", dialog, namespace="/hwr")

        return Response(status=200)

    return bp
